%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
-module(restapi).

%%%_* Exports ==========================================================
%% API
-export([ execute/1
        , execute/2
        , prepare/1
        , prepare/2
        , add_pool/7
        , result_to_array/1
        , result_to_obj/1
        , run_query/2
        , run_query/3
        , run_query_calc_rows/2
        ]).

%%%_* Inclues ==========================================================
-include("restapi.hrl").
-include_lib("emysql/include/emysql.hrl").

%%%_* Defines ==========================================================
-define(POOL, ?MODULE).

%%%_* API ==============================================================
-spec execute(atom()|list(), list()) -> term().
%% @doc Execute MySQL statement.
%% @end
execute([H|_] = StmtList, Args) when is_atom(H) ->
  [execute(Stmt, Args) || Stmt <- StmtList];
execute(Stmt, Args)                             ->
  emysql:execute(?POOL, Stmt, Args).

-spec execute(list()) -> list().
%% @doc Execute MySQL statements in sequance with the same
%% connection.
%% @end
execute([{_, _}|_] = Jobs) ->
  Timeout    = emysql_app:default_timeout(),
  Connection = emysql_conn_mgr:wait_for_connection(?POOL),
  Result     =
    [monitor_work(Connection, Timeout,
                  {emysql_conn, execute, [Connection, Stmt, Args]})
     || {Stmt, Args} <- Jobs],
  emysql_conn_mgr:pass_connection(Connection),
  Result.

-spec prepare(string()) -> ok.
%% @doc Prepare MySQL statement.
%% @end
prepare(Query) -> prepare(gen_id(), Query).

-spec prepare(atom(), string()|list()) -> ok.
%% @doc Prepare MySQL statement(s).
%% @end
prepare(Stmt, [[_|_]|_] = Queries) ->
  Ns = lists:seq(1, length(Queries)),
  [prepare(list_to_atom(atom_to_list(Stmt) ++ integer_to_list(N)), Q)
   || {N, Q} <- lists:zip(Ns, Queries)];
prepare(Stmt, Query)               ->
  emysql:prepare(Stmt, Query), Stmt.

-spec add_pool(integer(), string(), string(), string(), integer(),
               string(), atom()) -> ok.
%% @doc Add MySQL pool.
%% @end
add_pool(Size, Username, Password, Host, Port, Db, Encoding) ->
  emysql:add_pool(?POOL, Size, Username, Password, Host, Port, Db, Encoding).

%% @private
result_to_array(#result_packet{rows = []})                        -> [];
result_to_array(#result_packet{rows = [[_|_]|_] = Rows} = Result) ->
  case restapi_type:encode_values(emysql_util:field_names(Result)) of
    [_]     -> [restapi_type:encode(Value) || [Value] <- Rows];
    Fields  ->
      [ begin
          {struct, lists:zip(Fields, restapi_type:encode_values(Row))}
        end
        || Row <- Rows
      ]
  end;
result_to_array(#error_packet{} = Result)                         ->
  erlang:error(Result).

%% @private
result_to_obj(#result_packet{rows = []} = _Result)                  ->
  {struct, []};
result_to_obj(#result_packet{rows = [Row]} = Result)                ->
  Fields = restapi_type:encode_values(emysql_util:field_names(Result)),
  {struct, lists:zip(Fields, restapi_type:encode_values(Row))};
result_to_obj(#error_packet{} = Result)                             ->
  erlang:error(Result).

%% @private
run_query(Req, #state{id = Id, data = Data, argspec = ArgSpec} = State) ->
  Args = restapi_type:pick(Data, ArgSpec),
  run_query(Req, State#state{args = Args}, fun() -> execute(Id, Args) end).

%% @private
run_query_calc_rows(Req, #state{ id      = Id
                               , data    = Data
                               , argspec = ArgSpec
                               } = State) ->
  Args = restapi_type:pick(Data, ArgSpec),
  run_query(Req, State#state{args = Args},
            fun() ->
                execute([{Id, Args}, {<<"SELECT FOUND_ROWS();">>, []}])
            end).

%% @private
run_query(Req, #state{id = Id, args = Args, opts = O} = State, ExecF) ->
  F = fun() ->
          case ExecF() of
            Res when is_list(Res)                  ->
              case lists:keymember(error_packet, 1, Res) of
                true  -> erlang:error(Res);
                false -> {ok, Res}
              end;
            Res when is_record(Res, result_packet) -> {ok, Res};
            Res when is_record(Res, error_packet)  -> erlang:error(Res)
          end
      end,
  try restapi_cache:run({Id, Args}, F, orddict:fetch(cache_control, O))
  catch _Error:Rsn ->
      {PeerAddr, _} = cowboy_req:peer(Req),
      lager:error("error ~p returned in call from ~s: state=~p",
                  [ Rsn
                  , inet_parse:ntoa(PeerAddr)
                  , State
                  ]),
      erlang:error(Rsn)
  end.

%%%_* Internal =========================================================
gen_id() ->
  list_to_atom("restapi" ++ integer_to_list(erlang:phash2(erlang:now()))).

%% Modified work monitor. Do not pass the connection when finished.
%% Preserve as mush as possible of old behaviour.
%% See emysql:monitor_work/3
monitor_work(Connection, Timeout, {M, F, A})
  when is_record(Connection, emysql_connection) ->
  Parent = self(),
  Pid    =
    spawn(fun() ->
              receive start ->
                  Parent ! {self(), apply(M, F, A)}
              end
          end),
  Mref   = erlang:monitor(process, Pid),
  Pid ! start,
  receive
    {'DOWN', Mref, process, Pid, {_, closed}} ->
      case reset_connection(Connection, keep) of
        NewConnection when is_record(NewConnection, emysql_connection) ->
          [_OldConn|RestArgs] = A,
          NewA = [NewConnection|RestArgs],
          monitor_work(NewConnection, Timeout, {M, F, NewA});
        {error, FailedReset}                                           ->
          exit({connection_down, {and_conn_reset_failed, FailedReset}})
      end;
    {'DOWN', Mref, process, Pid, Reason}      ->
      case reset_connection(Connection, pass) of
        {error,FailedReset} ->
          exit({Reason, {and_conn_reset_failed, FailedReset}});
        _                   ->
          exit({Reason, {}})
      end;
    {Pid, Result}                             ->
      erlang:demonitor(Mref, [flush]),
      Result
  after Timeout ->
      erlang:demonitor(Mref),
      case reset_connection(Connection, pass) of
        {error, FailedReset} ->
          exit({mysql_timeout, Timeout, {and_conn_reset_failed, FailedReset}});
        _                    ->
          exit({mysql_timeout, Timeout, {}})
      end
  end.

reset_connection(Connection, Opt) ->
  emysql_conn:reset_connection(emysql_conn_mgr:pools(), Connection, Opt).

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
