%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi application
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
-module(restapi_app).

%%%_* Behaviour ========================================================
-behaviour(application).

%%%_* Exports ==========================================================
%% Application callbacks
-export([ start/2
        , stop/1
        ]).

%%%_* Application callbacks ============================================
-spec start(StartType::normal | {takeover, node()} | {failover, node()},
            StartArgs::term()) ->
               {ok, Pid::pid()}
             | {ok, Pid::pid(), State::term()}
             | {error, Reason::term()}.
%% @doc This function is called whenever an application is started using
%% application:start/1,2, and should start the processes of the
%% application. If the application is structured according to the OTP
%% design principles as a supervision tree, this means starting the top
%% supervisor of the tree.
%% @end
start(_StartType, _StartArgs) ->
  Envs = application:get_all_env(restapi),
  Lkup = fun(K) -> proplists:get_value(K, Envs) end,
  restapi:add_pool(Lkup(sql_connections),
                   Lkup(sql_username),
                   Lkup(sql_password),
                   Lkup(sql_host),
                   Lkup(sql_port),
                   Lkup(sql_database),
                   Lkup(sql_encoding)),
  ok = restapi_oauth2_backend:init(),
  APIDispatch =
    cowboy_router:compile(
      [ {'_', [ {"/v1/docs",
                 restapi_resource_docs, [{id, docs}]}
              , {"/v1/docs/:doc",
                 restapi_resource_docs, [{id, docs_x}]}
              , {"/v1/docs/:doc/file",
                 restapi_resource_file, [{id, docs_x_file}]}
              , {"/v1/authorities",
                 restapi_resource_authorities, [{id, authorities}]}
                , {"/v1/agents",
                 restapi_resource_agents, [{id, agents}]}
              , {"/v1/agents/:agent",
                 restapi_resource_agents, [{id, agents_x}]}
              , {"/v1/agents/:agent/targets",
                 restapi_resource_agents, [{id, agents_x_targets}]}
              , {"/v1/agents/:agent/targets/:target",
                 restapi_resource_agents, [{id, agents_x_targets_x}]}
              , {"/v1/auth",
                 restapi_resource_auth, [{id, auth}]}
              ]}
      ]),
  cowboy:start_http(
    restapi_http_listener, 100,
    [{port, Lkup(api_port)}],
    [{env, [{dispatch, APIDispatch}]}]),
  DeveloperDispatch =
    cowboy_router:compile(
      [ {'_', [ {"/", cowboy_static,
                 {priv_file, restapi, "www/index.html",
                  [{mimetypes, {<<"text">>, <<"html">>, []}}]}}
              , {"/[...]", cowboy_static,
                 {priv_dir, restapi, "www",
                  [{mimetypes, cow_mimetypes, web}]}}
              ]}
      ]),
  cowboy:start_http(
    developer_http_listener, 100,
    [{port, Lkup(developer_port)}],
    [{env, [{dispatch, DeveloperDispatch}]}]),
  MonitorDispatch =
    cowboy_router:compile(
      [ {'_', [ {"/ets", restapi_monitor_ets, []}
              , {"/memory", restapi_monitor_memory, []}
              , {"/statistics", restapi_monitor_statistics, []}
              , {"/process", restapi_monitor_process, []}
              , {"/system", restapi_monitor_system, []}
              , {"/metrics", restapi_monitor_metrics, []}
              , {"/metrics/[...]", restapi_monitor_metrics_list, []}
              , {"/alarms", restapi_monitor_alarms, []}
              ]}
      ]),
  cowboy:start_http(
    monitor_http_listener, 10,
    [{port, Lkup(monitor_port)}],
    [{env, [{dispatch, MonitorDispatch}]}]),
  restapi_sup:start_link().

-spec stop(State::term()) -> ok.
%% @doc This function is called whenever an application has stopped. It is
%% intended to be the opposite of Module:start/2 and should do any
%% necessary cleaning up.
%% @end
stop(_State) -> ok.

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
