%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi auth handler
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
%% @private
-module(restapi_auth_handler).

%%%_* Exports ==========================================================
%% API
-export([ is_authorized/2
        , forbidden/2
        ]).

%%%_* Includes =========================================================
-include_lib("restapi.hrl").

%%%_* API ==============================================================
is_authorized(Req, #state{id = Id, data = Data0} = State) ->
  case get_access_token(Req) of
    {ok, Token} ->
      case oauth2:verify_access_token(Token) of
        {ok, Context}          ->
          Identity = proplists:get_value(<<"identity">>, Context),
          Scope    = lists:keyfind(<<"scope">>, 1, Context),
          Data     = restapi_util:merge(
                       orddict:from_list([Identity, Scope]), Data0),
          case limited(Id) andalso is_over_limit(Identity) of
            true  ->
              Header = { <<"content-type">>
                       , <<"application/json; charset=utf-8">>
                       },
              Reason = <<"allowed number of daily requests for this resource "
                         "and client reached">>,
              Body   = restapi_json:encode([{<<"error">>, Reason}]),
              %% Maybe include a Retry-After header here?
              {ok, Reply} = cowboy_req:reply(429, [Header], Body, Req),
              {halt, Reply, State};
            false ->
              {true, Req, State#state{data = Data}}
          end;
        {error, access_denied} ->
          {{false, <<"Bearer">>}, Req, State}
      end;
    {error, _}  ->
      {{false, <<"Bearer">>}, Req, State}
  end.

forbidden(Req, #state{id = Id, data = Data} = State) ->
  Scope      = restapi_util:fetch(<<"scope">>, Data),
  ForbiddenP =
    not oauth2_priv_set:is_subset(oauth2_priv_set:new([Scope]),
                                  oauth2_priv_set:new(privliges(Id))),
  {ForbiddenP, Req, State}.

%%%_* Internal =========================================================
get_access_token(Req) ->
  case cowboy_req:header(<<"authorization">>, Req) of
    {<<"token ", Token/binary>>, _Req} -> {ok, Token};
    _                                   ->
      case cowboy_req:qs_val(<<"access_token">>, Req) of
        {undefined, _Req} -> {error, missing};
        {Token, _Req}     -> {ok, Token}
      end
  end.

is_over_limit({_IdentityType, Id}) ->
  N     = restapi_counter:get(Id),
  Limit = restapi_oauth2_backend:get_limit(Id),
  case Limit of
    unlimited             -> false;
    Limit when N >= Limit -> true;
    _                     ->
      restapi_counter:bump(Id, 86400), false
  end.

privliges(docs)               -> [<<"client.*">>, <<"user.*">>];
privliges(docs_nin)           -> [<<"client.*">>, <<"user.*">>];
privliges(docs_query)         -> [<<"client.*">>, <<"user.*">>];
privliges(docs_agent)         -> [<<"user.*">>];
privliges(docs_x)             -> [<<"client.*">>, <<"user.*">>];
privliges(docs_x_file)        -> [<<"client.*">>, <<"user.*">>];
privliges(authorities)        -> [<<"client.*">>, <<"user.*">>];
privliges(agents)             -> [<<"user.*">>];
privliges(agents_x)           -> [<<"user.*">>];
privliges(agents_x_targets)   -> [<<"user.*">>];
privliges(agents_x_targets_x) -> [<<"user.*">>].

limited(docs_x)      -> true;
limited(docs_x_file) -> true;
limited(_)           -> false.

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
