%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi cache
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
-module(restapi_cache).

%%%_* Behaviour ========================================================
-behaviour(gen_server).

%%%_* Exports ==========================================================
%% API
-export([ start_link/0
        , run/2
        , run/3
        , reset/0
        ]).

%% Gen server callbacks
-export([ init/1
        , handle_call/3
        , handle_cast/2
        , handle_info/2
        , terminate/2
        , code_change/3
        ]).

%% Tests
-export([ clean_cache_loop/3
        ]).

%%%_* Records ==========================================================
-record(cache, { id   :: term()    % Key
               , ttl  :: integer() % Time to live
               , data :: any()     % Data
               }).

%%%_* Defines ==========================================================
-define(SERVER,               ?MODULE).
-define(TABLE,                ?MODULE).
-define(CACHE_CLEAN_INTERVAL, 60).

%%%_* API ==============================================================
-spec start_link() -> {ok, Pid::pid()} | ignore | {error, Reason::term()}.
%% @doc Start server.
%% @end
start_link() -> gen_server:start_link({local, ?SERVER}, ?MODULE, [], []).

-spec run(Key::term(), QueryFun::function()) -> term().
%% @doc Get value from cache, or run question and populate cache.
%% Time-To-Live is set to one hour.
%% @end
run(Key, QueryFun) -> run(Key, QueryFun, 3600).

-spec run(Key::term(), QueryFun::function(), TTL::integer()|nocache) -> term().
%% @doc Get value from cache, or run question and populate cache.
%% Provide a Time-To-Live value (in seconds) for the cache entry.
%% The Time-To-Live value cannot be less than the
%% cache-clean-interval (60 seconds).
%% @end
run(_Key, QueryFun, nocache) ->
  case QueryFun() of
    {ok, Res} -> Res;
    Res       -> Res
  end;
run(Key,  QueryFun, TTL)     ->
  case TTL > ?CACHE_CLEAN_INTERVAL of
    true  ->
      case ets:lookup(?TABLE, Key) of
        []                   ->
          case QueryFun() of
            {ok, Res} ->
              ets:insert(?TABLE, #cache{ id   = Key
                                       , ttl  = ttl(TTL)
                                       , data = Res
                                       }),
              Res;
            Res       -> Res
          end;
        [#cache{data = Res}] -> Res
      end;
    false -> erlang:error(bad_ttl)
  end.

-spec reset() -> ok.
%% @doc Reset cache.
%% @end
reset() -> ets:delete_all_objects(?TABLE), ok.

%%%_* Gen server callbacks =============================================
%% @private
init([]) ->
  ets:new(?TABLE, [{keypos, 2}, public, named_table]),
  timer:send_interval(?CACHE_CLEAN_INTERVAL * 1000, clean_cache),
  {ok, []}.

%% @private
handle_call(_Msg, _From, State) -> {reply, ok, State}.

%% @private
handle_cast(_Msg, State) -> {noreply, State}.

%% @private
handle_info(clean_cache, State) ->
  clean_cache(),
  {noreply, State}.

%% @private
terminate(_Reason, _State) -> ok.

%% @private
code_change(_OldVsn, State, _Extra) -> {ok, State}.

%%%_* Internal =========================================================
clean_cache() -> clean_cache_loop(?TABLE, ets:first(?TABLE), timestamp()).

%% @private
clean_cache_loop(_Tab, '$end_of_table', _Now) -> ok;
clean_cache_loop(Tab,  Key,              Now) ->
  NextKey = ets:next(Tab, Key),
  case ets:lookup(Tab, Key) of
    [#cache{id = Key, ttl = TTL}] when TTL < Now ->
      ets:delete(Tab, Key),
      clean_cache_loop(Tab, NextKey, Now);
    _                                            ->
      clean_cache_loop(Tab, NextKey, Now)
  end.

ttl(TTL) -> timestamp() + TTL.

timestamp() -> calendar:datetime_to_gregorian_seconds(calendar:local_time()).

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
