%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi counter
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
-module(restapi_counter).

%%%_* Behaviour ========================================================
-behaviour(gen_server).

%%%_* Exports ==========================================================
%% API
-export([ start_link/0
        , bump/2
        , get/1
        , reset/0
        , reset/1
        ]).

%% Gen server callbacks
-export([ init/1
        , handle_call/3
        , handle_cast/2
        , handle_info/2
        , terminate/2
        , code_change/3
        ]).

%% Tests
-export([ clean_loop/3
        ]).

%%%_* Records ==========================================================
-record(counter, { id    :: term()            % Key
                 , ttl   :: non_neg_integer() % Time to live
                 , n = 1 :: non_neg_integer() % N
                 }).

%%%_* Defines ==========================================================
-define(SERVER,               ?MODULE).
-define(TABLE,                ?MODULE).
-define(CLEAN_INTERVAL, 60).

%%%_* API ==============================================================
-spec start_link() -> {ok, Pid::pid()} | ignore | {error, Reason::term()}.
%% @doc Start server.
%% @end
start_link() -> gen_server:start_link({local, ?SERVER}, ?MODULE, [], []).

-spec bump(Key::term(), TTL::non_neg_integer()) -> ok.
bump(Key, TTL) ->
  case ets:lookup(?TABLE, Key) of
    []                          ->
      true = ets:insert(?TABLE, #counter{id = Key, ttl = ttl(TTL)});
    [#counter{n = N} = Counter] ->
      true = ets:insert(?TABLE, Counter#counter{n = N + 1})
  end,
  ok.

-spec get(Key::term()) -> N::non_neg_integer().
get(Key) ->
  case ets:lookup(?TABLE, Key) of
    []                -> 0;
    [#counter{n = N}] -> N
  end.

-spec reset(Key::term()) -> ok.
%% @doc Reset counter.
%% @end
reset(Key) ->
  true = ets:delete(?TABLE, Key),
  ok.

-spec reset() -> ok.
%% @doc Reset all counters.
%% @end
reset() ->
  true = ets:delete_all_objects(?TABLE),
  ok.

%%%_* Gen server callbacks =============================================
%% @private
init([]) ->
  ets:new(?TABLE, [{keypos, 2}, public, named_table]),
  timer:send_interval(?CLEAN_INTERVAL * 1000, clean),
  {ok, []}.

%% @private
handle_call(_Msg, _From, State) -> {reply, ok, State}.

%% @private
handle_cast(_Msg, State) -> {noreply, State}.

%% @private
handle_info(clean, State) ->
  clean(),
  {noreply, State}.

%% @private
terminate(_Reason, _State) -> ok.

%% @private
code_change(_OldVsn, State, _Extra) -> {ok, State}.

%%%_* Internal =========================================================
clean() -> clean_loop(?TABLE, ets:first(?TABLE), timestamp()).

%% @private
clean_loop(_Tab, '$end_of_table', _Now) -> ok;
clean_loop(Tab,  Key,              Now) ->
  NextKey = ets:next(Tab, Key),
  case ets:lookup(Tab, Key) of
    [#counter{id = Key, ttl = TTL}] when TTL < Now ->
      true = ets:delete(Tab, Key),
      clean_loop(Tab, NextKey, Now);
    _                                              ->
      clean_loop(Tab, NextKey, Now)
  end.

ttl(TTL) -> timestamp() + TTL.

timestamp() -> calendar:datetime_to_gregorian_seconds(calendar:local_time()).

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
