%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi lager backend
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
%% @private
-module(restapi_lager_backend).

%%%_* Exports ==========================================================
%% API
-export([ subscribe/0
        , unsubscribe/0
        ]).

%% Gen event callbacks
-export([ init/1
        , handle_call/2
        , handle_event/2
        , handle_info/2
        , terminate/2
        , code_change/3
        ]).

%%%_* Records ==========================================================
-record(state, { loglevel
               , subscribers = []
               }).

%%%_* Define ===========================================================
-define(EVENT,   ?MODULE).
-define(TIMEOUT, 3000).

%%%_* API ==============================================================
subscribe() ->
  gen_event:call(lager_event, ?EVENT, {subscribe, self()}, 3000).

unsubscribe() ->
  gen_event:call(lager_event, ?EVENT, {unsubscribe, self()}, 3000).

%%%_* Gen event callbacks ==============================================
init(Args) ->
  Loglevel = proplists:get_value(loglevel, Args),
  {ok, #state{loglevel = Loglevel}}.

handle_call({subscribe, Pid}, #state{subscribers = S} = State)   ->
  {ok, ok, State#state{subscribers = [Pid|S]}};
handle_call({unsubscribe, Pid}, #state{subscribers = S} = State) ->
  {ok, ok, State#state{subscribers = lists:delete(Pid, S)}};
handle_call(get_loglevel, #state{loglevel = Loglevel} = State)   ->
  {ok, Loglevel, State};
handle_call({set_loglevel, Loglevel}, State)                     ->
  {ok, ok, State#state{loglevel = Loglevel}}.

handle_event({log, Msg}, #state{ loglevel    = Lvl
                               , subscribers = Subscribers0
                               } = State) ->
  Subscribers =
    case lager_msg:severity_as_int(Msg) =< lager_util:level_to_num(Lvl) of
      true  ->
        Message  = lager_msg:message(Msg),
        Severity = lager_msg:severity(Msg),
        case Subscribers0 of
          [] -> [];
          To ->
            Text = io_lib:format("~p: ~s", [Severity, Message]),
            [ begin
                Pid ! {self(), Severity, Text}, Pid
              end || Pid <- To, is_process_alive(Pid)
            ]
        end;
      false -> Subscribers0
    end,
  {ok, State#state{subscribers = Subscribers}};
handle_event(_Event, State)   -> {ok, State}.

handle_info(_Info, State) -> {ok, State}.

terminate(_Reason, _State) -> ok.

code_change(_OldVsn, State, _Extra) -> {ok, State}.

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
