%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi lager loogly backend
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
%% @private
-module(restapi_lager_loggly_backend).

%%%_* Exports ==========================================================
%% Gen event callbacks
-export([ init/1
        , handle_call/2
        , handle_event/2
        , handle_info/2
        , terminate/2
        , code_change/3
        ]).

%% Internal
-export([ deferred_log/3
        ]).

%%%_* Records ==========================================================
-record(state, { identity       :: string()
               , loglevel       :: non_neg_integer()
               , retry_interval :: non_neg_integer()
               , retry_times    :: non_neg_integer()
               , loggly_url     :: string()
               }).

%%%_* Gen events callbacks =============================================
init(Args) ->
  {ok, Identity}      = tulib_lists:assoc(identity, Args),
  {ok, RetryInterval} = tulib_lists:assoc(retry_interval, Args),
  {ok, RetryTimes}    = tulib_lists:assoc(retry_times, Args),
  {ok, LogglyUrl}     = tulib_lists:assoc(loggly_url, Args),
  {ok, Loglevel}      = tulib_lists:assoc(loglevel, Args),
  {ok, #state{ identity       = Identity
             , loglevel       = Loglevel
             , retry_interval = RetryInterval
             , retry_times    = RetryTimes
             , loggly_url     = LogglyUrl
             }}.

handle_call(get_loglevel, #state{loglevel=LogLevel} = State) ->
  {ok, LogLevel, State};
handle_call({set_loglevel, LogLevel}, State) ->
  {ok, ok, State#state{loglevel=LogLevel}}.

%% @private
handle_event({log, Msg},
             #state{ identity       = Identity
                   , loglevel       = Loglevel
                   , retry_interval = RetryInterval
                   , retry_times    = RetryTimes
                   , loggly_url     = LogglyUrl
                   } = State) when LogglyUrl =/= "" ->
  case lager_msg:severity_as_int(Msg) =< lager_util:level_to_num(Loglevel) of
    true  ->
      Message  = lager_msg:message(Msg),
      Severity = lager_msg:severity(Msg),
      PayLoad =
        restapi_json:encode(
          [ {<<"identity">>, restapi_type:to_binary(Identity)}
          , {<<"severity">>, restapi_type:to_binary(Severity)}
          , {<<"message">>,  restapi_type:to_binary(Message)}
          ]),
    Request = {LogglyUrl, [], "application/json", PayLoad},
    %% Spawn a background process to handle sending the payload.
    %% It will recurse until the payload has ben successfully sent.
    spawn(?MODULE, deferred_log, [Request, RetryTimes, RetryInterval]);
    false ->
      ok
  end,
  {ok, State};
handle_event(_Event, State) ->
  {ok, State}.

handle_info(_Info, State) ->
  {ok, State}.

terminate(_Reason, _State) ->
  ok.

code_change(_OldVsn, State, _Extra) ->
  {ok, State}.

%%%_* Internal =========================================================
deferred_log(_Request, 0, _)             -> ok;
deferred_log(Request, Retries, Interval) ->
  case httpc:request(post, Request, [],
                     [{body_format, binary}, {ssl, [{verify, 0}]}]) of
    {ok, {{_, 200, _}, _, _}} -> ok;
    _                         ->
      timer:sleep(Interval * 1000),
      deferred_log(Request, Retries - 1, Interval)
  end.

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
