%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi auth handler
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
%% @private
-module(restapi_oauth2_backend).

%%%_* Exports ==========================================================
%% API
-export([ init/0
        , new_client/0
        , add_client/2
        , add_client/3
        , delete_client/1
        , get_limit/1
        ]).

%% OAuth2 backend callbacks
-export([ authenticate_username_password/3
        , authenticate_client/3
        , associate_access_token/2
        , resolve_access_token/1
        , revoke_access_token/1
        ]).

%%%_* Includes =========================================================
-include_lib("restapi.hrl").
-include_lib("emysql/include/emysql.hrl").

%%%_* Defines ==========================================================
-define(ACCESS_TOKEN_TABLE, access_tokens).
-define(CLIENT_TABLE,       clients).
-define(TABLES,             [ ?ACCESS_TOKEN_TABLE
                            , ?CLIENT_TABLE
                            ]).

%%%_* Records ==========================================================
-record(client, { client_id         :: binary()
                , client_secret     :: binary()
                , limit = unlimited :: non_neg_integer()|unlimited
                }).

%%%_* API ==============================================================
init() ->
  lists:foreach(fun(Table) ->
                    ets:new(Table, [named_table, public])
                end, ?TABLES),
  case application:get_env(restapi, clients) of
    {ok, Clients} ->
      [ add_client( restapi_type:to_binary(Id)
                  , restapi_type:to_binary(Secret)
                  , Limit
                  )
        || {Id, Secret, Limit} <- Clients
      , is_integer(Limit) orelse Limit =:= unlimited
      ], ok;
    undefined     -> ok
  end.

new_client() ->
  ClientId     = generate_unique(),
  ClientSecret = generate_unique(),
  add_client(ClientId, ClientSecret),
  {ClientId, ClientSecret}.

add_client(ClientId, ClientSecret) ->
  add_client(ClientId, ClientSecret, unlimited).

add_client(ClientId, ClientSecret, Limit) ->
  put(?CLIENT_TABLE, ClientId, #client{ client_id     = ClientId
                                      , client_secret = ClientSecret
                                      , limit         = Limit
                                      }).

delete_client(ClientId) ->
  delete(?CLIENT_TABLE, ClientId).

get_limit(ClientId) ->
  case get(?CLIENT_TABLE, ClientId) of
    {ok, #client{limit = Limit}} -> Limit;
    {error, notfound}            ->
      case application:get_env(restapi, user_limit) of
        {ok, Limit} when is_integer(Limit) -> Limit;
        undefined                          -> 0
      end
  end.

%%%_* OAuth2 beckend callbacks =========================================
authenticate_username_password(Username, Password, _Scope) ->
  case get_user(Username) of
    {ok, {Id, UserPw, Salt, Deadline}} ->
      case pwhash(Password, Salt) of
        UserPw ->
          case is_active(Deadline) of
            true  -> {ok, {<<"user">>, Id}, <<"user.*">>};
            false -> {error, inactive}
          end;
        _      -> {error, badpass}
      end;
    {error, notfound} = Error          ->
      Error
  end.

authenticate_client(ClientId, ClientSecret, _Scope) ->
  case get(?CLIENT_TABLE, ClientId) of
    {ok, #client{client_secret = ClientSecret}} ->
      {ok, {<<"client">>, ClientId}, <<"client.*">>};
    {ok, #client{client_secret = _WrongSecret}} ->
      {error, badsecret};
    _                                           ->
      {error, notfound}
  end.

associate_access_token(AccessToken, Context) ->
  put(?ACCESS_TOKEN_TABLE, AccessToken, Context).

resolve_access_token(AccessToken) ->
  %% The case trickery is just here to make sure that
  %% we don't propagate errors that cannot be legally
  %% returned from this function according to the spec.
  case get(?ACCESS_TOKEN_TABLE, AccessToken) of
    {ok, _} = Value           -> Value;
    {error, notfound} = Error -> Error
  end.

revoke_access_token(AccessToken) ->
  delete(?ACCESS_TOKEN_TABLE, AccessToken).

%%%_* Internal =========================================================
%%%_* Access tokens & client table -------------------------------------
get(Table, Key) ->
  case ets:lookup(Table, Key) of
    []              -> {error, notfound};
    [{_Key, Value}] -> {ok, Value}
  end.

put(Table, Key, Value) ->
  ets:insert(Table, {Key, Value}),
  ok.

delete(Table, Key) ->
  ets:delete(Table, Key),
  ok.

%%%_* Users table ------------------------------------------------------
get_user(Username) ->
  restapi:prepare(
    auth,
    "SELECT id, password, salt, deadline "
    "FROM users WHERE email = ? AND blocked = '0';"),
  #result_packet{rows = [[Id, Password, Salt, Deadline]]} =
    restapi:execute(auth, [Username]),
  {ok, {Id, Password, Salt, Deadline}}.

pwhash(Str, Salt) ->
  {A, B} = lists:split(16, md5(restapi_type:to_list(Str))),
  restapi_type:to_binary(
    md5(A ++ ":" ++ restapi_type:to_list(Salt) ++ ":" ++ B)).

is_active(Deadline0) ->
  Deadline = restapi_type:to_list(restapi_type:encode(Deadline0)),
  Deadline =:= "0000-00-00" orelse Deadline > datestr().

md5(Str) ->
  lists:flatten(
    [io_lib:format("~2.16.0b", [N]) || N <- binary_to_list(erlang:md5(Str))]).

datestr() ->
  {Year, Month, Day} = erlang:date(),
  lists:flatten(io_lib:format("~4..0w-~2..0w-~2..0w", [Year, Month, Day])).

generate_unique() ->
  erlang:md5(integer_to_list(erlang:phash2(erlang:make_ref()))).

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
