%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi resource
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
%% @private
-module(restapi_resource).

%%%_* Exports ==========================================================
%% API
-export([ rest_init/2
        , uri_too_long/2
        , allowed_methods/2
        , malformed_request/2
        , is_authorized/2
        , forbidden/2
        , valid_entity_length/2
        , rest_terminate/2
        ]).

%%%_* Includes =========================================================
-include("restapi.hrl").

%%%_* API ==============================================================
rest_init(Req, Opts) ->
  lager:info("request: method=~s path=~s version=~p "
             "peer_address=~s peer_port=~p~n"
             "headers=~p~nbody=~p",
             [ untag(cowboy_req:method(Req))
             , untag(cowboy_req:path(Req))
             , untag(cowboy_req:version(Req))
             , inet_parse:ntoa(element(1, untag(cowboy_req:peer(Req))))
             , element(2, untag(cowboy_req:peer(Req)))
             , untag(cowboy_req:headers(Req))
             , untag(cowboy_req:body(Req))
             ]),
  {ok, Req, #state{ id        = proplists:get_value(id, Opts)
                  , timestamp = os:timestamp()
                  }}.

uri_too_long(Req, State) ->
  {URL, _} = cowboy_req:url(Req),
  {iolist_size(URL) > ?MAX_RAW_PATH_BYTES, Req, State}.

allowed_methods(Req, State) ->
  {[<<"GET">>], Req, State}.

malformed_request(Req, State) ->
  type_check(Req, State).

is_authorized(Req, State) ->
  restapi_auth_handler:is_authorized(Req, State).

forbidden(Req, State) ->
  restapi_auth_handler:forbidden(Req, State).

valid_entity_length(Req, State) ->
  {Length, _} = cowboy_req:body_length(Req),
  {Length < ?MAX_BODY_BYTES, Req, State}.

rest_terminate(Req, #state{timestamp = Timestamp}) ->
  FTimestamp = os:timestamp(),
  lager:info(
    "call ~p from ~s: started=~s ended=~s duration=~p ms",
    [ untag(cowboy_req:path(Req))
    , inet_parse:ntoa(element(1, untag(cowboy_req:peer(Req))))
    , restapi_util:format_timestamp(Timestamp)
    , restapi_util:format_timestamp(FTimestamp)
    , restapi_util:timestamp_to_ms(FTimestamp) -
        restapi_util:timestamp_to_ms(Timestamp)
    ]),
  ok.

%%%_* Internal =========================================================
%%%_* Type checking ----------------------------------------------------
type_check(Req, #state{argspec = ArgSpec} = State) ->
  try
    Data = restapi_type:check(get_req_args(Req), ArgSpec),
    {false, Req, State#state{data = Data}}
  catch Error:Rsn ->
      lager:info(
        "malformed request from ~s: error=~p reason=~p "
        "state=~p stacktrace=~p",
        [ inet_parse:ntoa(element(1, untag(cowboy_req:peer(Req))))
        , Error
        , Rsn
        , State
        , erlang:get_stacktrace()
        ]),
      {true, Req, State}
  end.

get_req_args(Req) ->
  case untag(cowboy_req:method(Req)) of
    <<"POST">>   -> get_all_args(Req);
    <<"GET">>    -> get_url_args(Req);
    <<"PUT">>    -> get_all_args(Req);
    <<"DELETE">> -> get_url_args(Req)
  end.

get_all_args(Req) ->
  restapi_util:merge(get_body_args(Req), get_url_args(Req)).

get_body_args(Req) ->
  case cowboy_req:body(Req) of
    {ok, <<>>, _} -> orddict:new();
    {ok, Body, _} ->
      case mochijson2:decode(Body) of
        {struct, JSON} when is_list(JSON) -> orddict:from_list(JSON);
        _                                 -> throw(malformed)
      end
  end.

get_url_args(Req) ->
  orddict:from_list(
    lists:append(
      [{atom_to_binary(K, utf8), V}
       || {K, V} <- element(1, cowboy_req:bindings(Req))],
      [{K, V}
       || {K, V} <- element(1, cowboy_req:qs_vals(Req))])).

untag({ok, Value, _Req}) -> Value;
untag({Value, _Req})     -> Value.

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
