%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi agents resource
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
%% @private
-module(restapi_resource_agents).

%%%_* Exports ==========================================================
%% API
-export([ init/3
        , rest_init/2
        , uri_too_long/2
        , allowed_methods/2
        , malformed_request/2
        , is_authorized/2
        , forbidden/2
        , valid_entity_length/2
        , content_types_provided/2
        , charsets_provided/2
        , get_json/2
        , rest_terminate/2
        ]).

%%%_* Includes =========================================================
-include("restapi.hrl").
-include_lib("emysql/include/emysql.hrl").

%%%_* API ==============================================================
init(_Transport, _Req, _Opts) ->
  {upgrade, protocol, cowboy_rest}.

rest_init(Req, Opts) ->
  restapi_resource:rest_init(Req, Opts).

uri_too_long(Req, State) ->
  restapi_resource:uri_too_long(Req, State).

allowed_methods(Req, State) ->
  restapi_resource:allowed_methods(Req, State).

malformed_request(Req, State) ->
  restapi_resource:malformed_request(Req, argspec(Req, State)).

is_authorized(Req, State) ->
  restapi_resource:is_authorized(Req, State).

forbidden(Req, State) ->
  restapi_resource:forbidden(Req, State).

valid_entity_length(Req, State) ->
  restapi_resource:valid_entity_length(Req, State).

content_types_provided(Req, State) ->
  {[{{<<"application">>, <<"json">>, []}, get_json}], Req, State}.

charsets_provided(Req, State) ->
  {[<<"utf-8">>], Req, State}.

get_json(Req, #state{id = Id0} = State) ->
  Statement = statement(Req, Id0),
  Opts      =
    [ {cache_control, 900}
    ],
  Id = restapi:prepare(Id0, element(1, Statement)),
  get_json(Req, State#state{id = Id, opts = Opts}, Id0).

rest_terminate(Req, State) ->
  restapi_resource:rest_terminate(Req, State).

%%%_* Internal =========================================================
get_json(Req, State, agents)                            ->
  [Res, #result_packet{rows = [[Count]]}] =
    restapi:run_query_calc_rows(Req, State),
  L = [ {count, Count}
      , {agents, restapi:result_to_array(Res)}
      ],
  {JSON, Reply} = restapi_json:make(Req, {struct, L}),
  {JSON, Reply, State};
get_json(Req, State, agents_x_targets)                  ->
  [Res, #result_packet{rows = [[Count]]}] =
    restapi:run_query_calc_rows(Req, State),
  L = [ {count, Count}
      , {targets, restapi:result_to_array(Res)}
      ],
  {JSON, Reply} = restapi_json:make(Req, {struct, L}),
  {JSON, Reply, State};
get_json(Req, State, Id) when Id =:= agents_x
                            ; Id =:= agents_x_targets_x ->
  Res = restapi:run_query(Req, State),
  case restapi:result_to_obj(Res) of
    {struct, []} ->
      {ok, Reply} = cowboy_req:reply(404, Req),
      {<<>>, Reply, State};
    JSON0        ->
      {JSON, Reply} = restapi_json:make(Req, JSON0),
      {JSON, Reply, State}
  end.

argspec(Req, #state{id = Id} = State) ->
  State#state{argspec = element(2, statement(Req, Id))}.

%% Requires user as argument. Gets appended before
%% making query.
statement(_Req, agents)             ->
  { "SELECT SQL_CALC_FOUND_ROWS "
    "agents_batch.id, agents_batch.ab_title AS title "
    "FROM agents_batch LEFT JOIN agents_follow ON "
    "agents_follow.batch=agents_batch.id "
    "WHERE agents_follow.user = ? ORDER BY agents_batch.ab_title;"
  , [ {'user', no_validation}
    ]
  };
statement(_Req, agents_x)           ->
  { "SELECT agents_batch.id, agents_batch.ab_title AS title, "
    "CASE WHEN agents_follow.send_email > 0 THEN 'true' ELSE 'false' END "
    "AS send_email, "
    "CASE WHEN agents_batch.ab_type = 1 THEN 'nin' ELSE 'freetext' END "
    "AS type "
    "FROM agents_batch LEFT JOIN "
    "agents_follow ON agents_follow.batch=agents_batch.id WHERE "
    "agents_batch.id = ? AND agents_follow.user = ?;"
  , [ {'agent', integer}
    , {'user', no_validation}
    ]
  };
statement(_Req, agents_x_targets)   ->
  { "SELECT SQL_CALC_FOUND_ROWS "
    "agents.id, agents.target, agents.comment FROM agents LEFT JOIN "
    "agents_batch ON agents.batch=agents_batch.id LEFT JOIN "
    "agents_follow ON agents_follow.batch=agents_batch.id WHERE "
    "agents.batch = ? AND agents_follow.user = ?;"
  , [ {'agent', integer}
    , {'user', no_validation}
    ]
  };
statement(_Req, agents_x_targets_x) ->
  { "SELECT agents.id, agents.target, agents.comment FROM agents "
    "LEFT JOIN agents_batch ON agents.batch=agents_batch.id "
    "LEFT JOIN agents_follow ON agents_follow.batch=agents_batch.id "
    "WHERE agents.id = ? AND agents_follow.user = ?;"
  , [ {'target', integer}
    , {'user', no_validation}
    ]
  }.

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
