%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi resource auth
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
%% @private
-module(restapi_resource_auth).

%%%_* Exports ==========================================================
%% API
-export([ init/3
        , rest_init/2
        , uri_too_long/2
        , allowed_methods/2
        , valid_entity_length/2
        , content_types_provided/2
        , content_types_accepted/2
        , charsets_provided/2
        , process_post/2
        , rest_terminate/2
        ]).

%%%_* Includes =========================================================
-include_lib("restapi.hrl").

%%%_* API ==============================================================
init(_Transport, _Req, _Opts) ->
  {upgrade, protocol, cowboy_rest}.

rest_init(Req, Opts) ->
  restapi_resource:rest_init(Req, Opts).

uri_too_long(Req, State) ->
  restapi_resource:uri_too_long(Req, State).

allowed_methods(Req, State) ->
  {[<<"POST">>], Req, State}.

valid_entity_length(Req, State) ->
  restapi_resource:valid_entity_length(Req, State).

content_types_provided(Req, State) ->
  {[ {{<<"application">>, <<"json">>, []}, process_post}
   ], Req, State}.

content_types_accepted(Req, State) ->
  {[ {{<<"application">>, <<"x-www-form-urlencoded">>, []}, process_post}
   ], Req, State}.

charsets_provided(Req, State) ->
  {[<<"utf-8">>], Req, State}.

process_post(Req, State) ->
  {ok, Body, _Req} = cowboy_req:body(Req),
  Params           = decode(Body),
  {ok, Reply}      =
    case lists:max([proplists:get_value(K, Params)
                    || K <- [<<"grant_type">>, <<"response_type">>]]) of
      <<"password">>           ->
        process_password_grant(Req, Params);
      <<"client_credentials">> ->
        process_client_credentials_grant(Req, Params);
      _                        ->
        cowboy_req:reply(400, [], <<>>, Req)
    end,
  {halt, Reply, State}.

rest_terminate(Req, State) ->
  restapi_resource:rest_terminate(Req, State).

%%%_* Internal =========================================================
%%%_* Grant type handlers ----------------------------------------------
process_password_grant(Req, Params) ->
  Username = proplists:get_value(<<"username">>, Params),
  Password = proplists:get_value(<<"password">>, Params),
  Scope    = proplists:get_value(<<"scope">>, Params, <<"">>),
  response(oauth2:authorize_password(Username, Password, Scope), Req).

process_client_credentials_grant(Req, Params) ->
  {<<"Basic ", Credentials/binary>>, _Req} =
    cowboy_req:header(<<"authorization">>, Req),
  [Id, Secret] = binary:split(base64:decode(Credentials), <<":">>),
  Scope        = proplists:get_value(<<"scope">>, Params, <<"">>),
  response(oauth2:authorize_client_credentials(Id, Secret, Scope), Req).

%%%_* Helpers ----------------------------------------------------------
response(AuthResult, Req) ->
  {Code, JSON} =
    case AuthResult of
      {ok, _Identity, Response} ->
        {200, restapi_json:encode(
                lists:filter(fun({_, <<>>}) -> false;
                                (_)         -> true
                             end, oauth2_response:to_proplist(Response)))};
      {error, Reason}           ->
        {PeerAddr, _} = cowboy_req:peer(Req),
        lager:info("unauthorized request from ~s with error ~p",
                   [ inet_parse:ntoa(PeerAddr)
                   , Reason
                   ]),
        {401, restapi_json:encode(
                [{<<"error">>, restapi_type:to_binary(Reason)}])}
    end,
  cowboy_req:reply(Code, [], JSON, Req).

decode(Body) ->
  RawBody = cowboy_http:urldecode(Body),
  Pairs   = binary:split(RawBody, <<"&">>, [global]),
  lists:map(fun(Pair) ->
                [K, V] = binary:split(Pair, <<"=">>),
                {K, V}
            end,
            Pairs).

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
