%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi docs resource
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
%% @private
-module(restapi_resource_docs).

%%%_* Exports ==========================================================
%% API
-export([ init/3
        , rest_init/2
        , uri_too_long/2
        , allowed_methods/2
        , malformed_request/2
        , is_authorized/2
        , forbidden/2
        , valid_entity_length/2
        , content_types_provided/2
        , charsets_provided/2
        , get_json/2
        , rest_terminate/2
        ]).

%%%_* Includes =========================================================
-include("restapi.hrl").
-include_lib("emysql/include/emysql.hrl").

%%%_* API ==============================================================
init(_Transport, _Req, _Opts) ->
  {upgrade, protocol, cowboy_rest}.

rest_init(Req, Opts) ->
  restapi_resource:rest_init(Req, Opts).

uri_too_long(Req, State) ->
  restapi_resource:uri_too_long(Req, State).

allowed_methods(Req, State) ->
  restapi_resource:allowed_methods(Req, State).

malformed_request(Req, State) ->
  restapi_resource:malformed_request(Req, argspec(Req, State)).

is_authorized(Req, State) ->
  restapi_resource:is_authorized(Req, State).

forbidden(Req, State) ->
  restapi_resource:forbidden(Req, State).

valid_entity_length(Req, State) ->
  restapi_resource:valid_entity_length(Req, State).

content_types_provided(Req, State) ->
  {[{{<<"application">>, <<"json">>, []}, get_json}], Req, State}.

charsets_provided(Req, State) ->
  {[<<"utf-8">>], Req, State}.

get_json(Req, #state{id = Id0} = State) ->
  Statement = statement(Req, Id0),
  Opts      =
    [ {cache_control, 3600}
    ],
  Id = restapi:prepare(Id0, element(1, Statement)),
  get_json(Req, State#state{id = Id, opts = Opts}, Id0).

rest_terminate(Req, State) ->
  restapi_resource:rest_terminate(Req, State).

%%%_* Internal =========================================================
get_json(Req, State, Id) when Id =:= docs
                            ; Id =:= docs_query
                            ; Id =:= docs_nin
                            ; Id =:= docs_agent ->
  [Res, #result_packet{rows = [[Count]]}] =
    restapi:run_query_calc_rows(Req, State),
  L = [ {count, Count}
      , {documents, restapi:result_to_array(Res)}
      ],
  {JSON, Reply} = restapi_json:make(Req, {struct, L}),
  {JSON, Reply, State};
get_json(Req, State, docs_x)                  ->
  [Doc, Actors, Authorities] = restapi:run_query(Req, State),
  case restapi:result_to_obj(Doc) of
    {struct, []} ->
      {ok, Reply} = cowboy_req:reply(404, Req),
      {halt, Reply, State};
    {struct, L0} ->
      L = lists:reverse(
            [ {actors, restapi:result_to_array(Actors)}
            , {affected_authorities, restapi:result_to_array(Authorities)}
            | L0
            ]),
      {JSON, Reply0} = restapi_json:make(Req, {struct, L}),
      Reply = cowboy_req:set_resp_header( <<"cache-control">>
                                        , <<"public">>
                                        , Reply0
                                        ),
      {JSON, Reply, State}
  end.

argspec(Req, State) ->
  argspec_aux([<<"agent">>, <<"query">>, <<"nin">>], Req, State).

argspec_aux([], Req, #state{id = Id} = State)       ->
  State#state{argspec = element(2, statement(Req, Id))};
argspec_aux([Key|T], Req, #state{id = Id0} = State) ->
  case cowboy_req:qs_val(Key, Req) of
    {undefined, _}               -> argspec_aux(T, Req, State);
    {Val, _} when is_binary(Val) ->
      Id = list_to_atom(atom_to_list(Id0) ++ "_" ++ binary_to_list(Key)),
      State#state{ id      = Id
                 , argspec = element(2, statement(Req, Id))
                 }
  end.

statement(Req, docs)        ->
  { "SELECT SQL_CALC_FOUND_ROWS "
    "document.historic AS historic_date, document.actors, "
    "document.fsize AS file_size, document.md5sum AS id, document.filename, "
    "document.ocr_date AS date, document.pages, "
    "document.myndighet AS authority_id, "
    "myndigheter.namn AS authority_name "
    "FROM document LEFT JOIN myndigheter ON "
    "myndigheter.id=document.myndighet WHERE "
    "DATE(document.ocr_date) >= ? "
    "AND DATE(document.ocr_date) <= ? "
    ++ maybe_authority(Req) ++
    "ORDER BY ocr_date DESC LIMIT ?,50;"
  , [ {'begin-date', date, "0000-00-00"}
    , {'end-date', date, timestamp}
    , {'authority', integer, optional}
    , {'start-index', integer, 0}
    ]
  };
%% NOTE: If a individual word is in more than 50% of rows, it is treated
%% as a "stopword", returing a relevance (score) of zero.
statement(Req, docs_query)  ->
  { "SELECT SQL_CALC_FOUND_ROWS "
    "document.historic AS historic_date, document.actors, "
    "document.fsize AS file_size, document.md5sum AS id, document.filename, "
    "document.ocr_date AS date, document.pages, "
    "document.myndighet AS authority_id, "
    "myndigheter.namn AS authority_name, "
    "MATCH(d_cleartext) AGAINST (?) AS score "
    "FROM document LEFT JOIN myndigheter ON "
    "myndigheter.id=document.myndighet WHERE MATCH(d_cleartext) AGAINST "
    "(? IN BOOLEAN MODE) AND DATE(document.ocr_date) >= ? "
    "AND DATE(document.ocr_date) <= ? "
    ++ maybe_authority(Req) ++
    "ORDER BY score DESC LIMIT ?,50;"
  , [ {'query', fun query_to_bool/1, ""}
    , {'query', fun query_to_bool/1, ""}
    , {'begin-date', date, "0000-00-00"}
    , {'end-date', date, timestamp}
    , {'authority', integer, optional}
    , {'start-index', integer, 0}
    ]
  };
statement(Req, docs_nin)    ->
  { "SELECT SQL_CALC_FOUND_ROWS "
    "document.historic AS historic_date, document.actors, "
    "document.fsize AS file_size, document.md5sum AS id, "
    "document.filename, document.ocr_date AS date, document.pages, "
    "document.myndighet AS authority_id, "
    "myndigheter.namn AS authority_name "
    "FROM d_pnr LEFT JOIN document ON d_pnr.document=document.md5sum "
    "LEFT JOIN myndigheter ON myndigheter.id=document.myndighet "
    "WHERE pnr = ? AND fsize > 0 AND DATE(document.ocr_date) >= ? "
    "AND DATE(document.ocr_date) <= ? "
    ++ maybe_authority(Req) ++
    "ORDER BY ocr_date DESC LIMIT ?,50;"
  , [ {'nin', <<"^[0-9]{6}-[0-9]{4}$">>}
    , {'begin-date', date, "0000-00-00"}
    , {'end-date', date, timestamp}
    , {'authority', integer, optional}
    , {'start-index', integer, 0}
    ]
  };
statement(Req, docs_agent)  ->
  { "SELECT SQL_CALC_FOUND_ROWS "
    "document.historic AS historic_date, document.actors, "
    "document.fsize AS file_size, document.md5sum AS id, "
    "document.filename, document.ocr_date AS date, document.pages, "
    "document.myndighet AS authority_id, "
    "myndigheter.namn AS authority_name "
    "FROM agents_results LEFT JOIN agents ON "
    "agents_results.agent=agents.id LEFT JOIN "
    "agents_batch ON agents.batch=agents_batch.id LEFT JOIN "
    "agents_follow ON agents_follow.batch=agents_batch.id "
    "LEFT JOIN document ON agents_results.document=document.md5sum "
    "LEFT JOIN myndigheter ON myndigheter.id=document.myndighet "
    "WHERE agents_follow.user = ? AND agents_batch.id = ? AND "
    "DATE(document.ocr_date) >= ? "
    "AND DATE(document.ocr_date) <= ? "
    ++ maybe_authority(Req) ++
    "GROUP BY document.serial ORDER BY ocr_date DESC LIMIT ?,50;"
  , [ {'user', no_validation}
    , {'agent', integer}
    , {'begin-date', date, "0000-00-00"}
    , {'end-date', date, timestamp}
    , {'authority', integer, optional}
    , {'start-index', integer, 0}
    ]
  };
statement(_Req, docs_x)     ->
  { [ "SELECT document.historic AS historic_date, "
      "document.fsize AS file_size, document.md5sum AS id, "
      "document.filename, "
      "document.ocr_date AS date, document.pages, "
      "document.d_cleartext AS cleartext, "
      "document.myndighet AS authority_id, "
      "myndigheter.namn AS authority_name "
      "FROM document LEFT JOIN myndigheter ON "
      "myndigheter.id=document.myndighet WHERE document.md5sum = ?;"
    , "SELECT d_pnr.pnr AS actors FROM d_pnr WHERE document = ?;"
    , "SELECT myndigheter.namn AS affected_authorities FROM d_mynd "
      "LEFT JOIN myndigheter ON d_mynd.mynd=myndigheter.id "
      "WHERE document = ? ORDER BY namn;"
    ]
  , [ {'doc', string}
    ]
  }.

maybe_authority(Req) ->
  case cowboy_req:qs_val(<<"authority">>, Req) of
    {undefined, _Req} -> "";
    {_Else, _Req}     -> "AND document.myndighet = ? "
  end.

%%%_* Query to boolean full text search --------------------------------
%% http://dev.mysql.com/doc/refman/5.5/en//fulltext-boolean.html
query_to_bool(Str) when is_binary(Str) -> query_to_bool(binary_to_list(Str));
query_to_bool(Str) when is_list(Str)   ->
  list_to_binary(
    lists:flatten(
      lists:map(
        fun(Val) ->
            RegExpOpts = [{capture, first, list}, caseless],
            case re:run(Val, "^[\(](.*)[\)]$", RegExpOpts) of
              {match, [Val]} ->
                " +(" ++ prefixx(string:centre(Val, length(Val) - 2)) ++ ")";
              nomatch        ->
                case re:run(Val, " ", [{capture, none}, caseless]) of
                  match   ->  " +\"" ++ Val ++ "\"";
                  nomatch -> prefixx(Val)
              end
            end
        end, explode_respect_quotes(Str)))).

prefixx(Str) ->
  lists:map(
    fun(Val) ->
        case re:run(Val, "^-", [{capture, none}]) of
          match   -> " " ++ Val;
          nomatch -> " +" ++ Val
        end
    end, string:tokens(Str, " ")).

explode_respect_quotes(Str0) ->
  case re:run(Str0, "^\".*\"$", [{capture, none}]) of
    match   -> [Str0];
    nomatch ->
      Str1 = explode_respect_quotes_aux(Str0, 0, []),
      Str  = [C || C <- Str1, C =/= $\\],
      string:tokens(Str, "~~~~") % Magic key
  end.

explode_respect_quotes_aux([], _QuoteLevel, Acc)    ->
  lists:reverse(lists:flatten(Acc));
explode_respect_quotes_aux([$"|T], QuoteLevel, Acc) ->
  case QuoteLevel + 1 of
    2 -> explode_respect_quotes_aux(T, 0, Acc);
    Q -> explode_respect_quotes_aux(T, Q, Acc)
  end;
explode_respect_quotes_aux([H|T], QuoteLevel, Acc)  ->
  case H =:= $\  andalso QuoteLevel =:= 0 of
    true  -> explode_respect_quotes_aux(T, QuoteLevel, ["~~~~"|Acc]);
    false -> explode_respect_quotes_aux(T, QuoteLevel, [H|Acc])
  end.

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
