%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi resource file
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
%% @private
-module(restapi_resource_file).

%%%_* Exports ==========================================================
-export([ init/3
        , rest_init/2
        , uri_too_long/2
        , allowed_methods/2
        , malformed_request/2
        , is_authorized/2
        , forbidden/2
        , valid_entity_length/2
        , content_types_provided/2
        , rest_terminate/2
        ]).

%%%_* Inclues ==========================================================
-include("restapi.hrl").
-include_lib("emysql/include/emysql.hrl").

%%%_* API ==============================================================
init(_Transport, _Req, _Opts) ->
  {upgrade, protocol, cowboy_rest}.

rest_init(Req, Opts) ->
  restapi_resource:rest_init(Req, Opts).

uri_too_long(Req, State) ->
  restapi_resource:uri_too_long(Req, State).

allowed_methods(Req, State) ->
  restapi_resource:allowed_methods(Req, State).

malformed_request(Req, #state{id = Id} = State) ->
  restapi_resource:malformed_request(
    Req, State#state{argspec = element(2, statement(Id))}).

is_authorized(Req, State) ->
  restapi_resource:is_authorized(Req, State).

forbidden(Req, State) ->
  restapi_resource:forbidden(Req, State).

valid_entity_length(Req, State) ->
  restapi_resource:valid_entity_length(Req, State).

content_types_provided(Req, State) ->
  {ok, Reply} =
    case get_file(Req, State) of
      <<>>                                      -> cowboy_req:reply(404, Req);
      {Filename0, FileSize, Compressed, Binary} ->
        Filename = binary_to_list(Filename0),
        Body     = if Compressed =:= 1 -> inflate(Binary); true -> Binary end,
        Headers  =
          [ {<<"cache-control">>, <<"public">>}
          , {<<"content-description">>, <<"File Transfer">>}
          , {<<"content-disposition">>,
             list_to_binary("attachment; filename=" ++ Filename)}
          , {<<"content-type">>,
             list_to_binary(from_extension(filename:extension(Filename)))}
          , {<<"Content-Transfer-Encoding">>, <<"Binary">>}
          , {<<"content-length">>, integer_to_list(FileSize)}
          ],
        cowboy_req:reply(200, Headers, Body, Req)
    end,
  {halt, Reply, State}.

rest_terminate(Req, State) ->
  restapi_resource:rest_terminate(Req, State).

%%%_* Internal =========================================================
get_file(Req0, #state{id = Id} = State) ->
  restapi:prepare(Id, element(1, statement(Id))),
  do_get_file(Req0, State#state{opts = [{cache_control, 3600}]}).

do_get_file(Req, #state{ id      = Id
                       , data    = Data
                       , argspec = ArgSpec
                       , opts    = O
                       } = State) ->
  try do_query(Id, restapi_type:pick(Data, ArgSpec), O)
  catch _Error:Rsn ->
      {PeerAddr, _} = cowboy_req:peer(Req),
      lager:error("error returned in call from ~s: state=~p",
                  [ PeerAddr
                  , State
                  ]),
      erlang:error(Rsn)
  end.

do_query(Id, Args, O) ->
  F = fun() ->
          result(restapi:execute(Id, Args))
      end,
  restapi_cache:run({Id, Args}, F, orddict:fetch(cache_control, O)).

result(#result_packet{rows = []})                                         ->
  {ok, <<>>};
result(#result_packet{rows = [[Filename, FileSize, Compressed, Binary]]}) ->
  {ok, {Filename, FileSize, Compressed, Binary}};
result(Result)                                                            ->
  {error, Result}.

from_extension(".docx") ->
  "application/vnd.openxmlformats-officedocument.wordprocessingml.document";
from_extension(".doc")  -> "application/vnd.ms-word";
from_extension(".xls")  -> "application/vnd.ms-excel";
from_extension(".xps")  -> "application/vnd.ms-xpsdocument";
from_extension(".ppt")  -> "application/vnd.ms-powerpoint";
from_extension(".tif")  -> "image/tiff";
from_extension(".tiff") -> "image/tiff";
from_extension(Ext)     -> mochiweb_mime:from_extension(Ext).

statement(docs_x_file) ->
  { "SELECT document.filename, document.fsize, d_pdf.d_compressed, "
    "d_pdf.d_blob FROM document "
    "LEFT JOIN d_pdf ON document.md5sum=d_pdf.d_md5 WHERE "
    "d_md5 = ?;"
  , [ {doc, string}
    ]
  }.

inflate(Binary) ->
  Z  = zlib:open(),
  ok = zlib:inflateInit(Z, -15),
  DeCompressed = zlib:inflate(Z, Binary),
  ok = zlib:inflateEnd(Z),
  ok = zlib:close(Z),
  iolist_to_binary(DeCompressed).

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
