%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi type
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
-module(restapi_type).

%%%_* Exports ==========================================================
%% API
-export([ check/2
        , pick/2
        , valid/1
        , encode_values/1
        , encode/1
        , to_list/1
        , to_binary/1
        ]).

%%%_* API ==============================================================
-spec check(list(), list()) -> list().
%% @doc Typecheck arguments.
%% @end
check(Orddict, ArgSpec) -> do_check(Orddict, ArgSpec).

-spec pick(list(), list()) -> list().
%% @doc Pick arguments into a valid argument list.
%% @end
pick(Orddict, ArgSpec) -> do_pick(Orddict, ArgSpec).

-spec valid(list()) -> boolean().
%% @doc Check argument specification validity.
%% @end
valid(ArgSpec) when is_list(ArgSpec) ->
  lists:all(fun({Arg, Type})       when is_atom(Arg) ->
                lists:member(Type, types());
               ({Arg, Type, _Def}) when is_atom(Arg) ->
                lists:member(Type, types());
               (_)                                   -> false
            end, ArgSpec);
valid(_ArgSpec)                      -> false.

-spec encode_values([term()]) -> list().
%% @doc Encode list of EMySQL terms so they can be safley encoded in
%% mochijson2:encode/1.
%% @end
encode_values(L) -> [encode(Val) || Val <- L].

-spec encode(term()) -> list().
%% @doc Encode EMySQL term so it can be safley encoded in mochijson2:encode/1.
%% @end
encode(Val) ->
  case is_binary(Val) of
    true  ->
      {Encoding, _} = unicode:bom_to_encoding(Val),
      encode(Val, Encoding);
    false -> encode(Val, latin1)
  end.

-spec to_list(binary()|list()|float()|atom()|integer()) -> list().
%% @doc To list.
%% @end
to_list(X) when is_integer(X) -> integer_to_list(X);
to_list(X) when is_binary(X)  -> binary_to_list(X);
to_list(X) when is_atom(X)    -> atom_to_list(X);
to_list(X) when is_float(X)   -> mochinum:digits(X);
to_list(X) when is_list(X)    -> X.

-spec to_binary(binary()|list()|float()|atom()|integer()) -> binary().
%% @doc To binary.
%% @end
to_binary(X) -> unicode:characters_to_binary(to_list(X)).

%%%_* Internal =========================================================
do_check(Orddict, Spec) ->
  lists:foldl(
    fun({_Key, no_validation}, Acc) -> Acc; % Value not from request
       ({Key, Type},           Acc) ->
        orddict:store( to_binary(Key)
                     , validate(restapi_util:fetch(Key, Orddict), Type)
                     , Acc
                     );
       ({Key, Type, Default},  Acc) ->
        case restapi_util:fetch(Key, Orddict, Default) of
          optional -> Acc;
          Value    ->
            orddict:store(to_binary(Key), validate(Value, Type), Acc)
        end
    end, orddict:new(), Spec).

do_pick(Orddict, Spec) ->
  lists:reverse(
    lists:foldl(fun({Key, _Type, Default}, Acc) ->
                    case restapi_util:fetch(Key, Orddict, Default) of
                      optional -> Acc;
                      Value    -> [Value|Acc]
                    end;
                   ({Key, _Type}, Acc)          ->
                    [restapi_util:fetch(Key, Orddict)|Acc]
                end, [], Spec)).

%% Incoming requests
validate(X, float) when is_float(X)     -> X;
validate(X, float) when is_list(X)      -> mochinum:digits(X);
validate(X, float) when is_binary(X)    ->
  validate(binary_to_list(X), float);
validate(X, integer) when is_integer(X) -> X;
validate(X, integer) when is_list(X)    -> list_to_integer(X);
validate(X, integer) when is_binary(X)  ->
  validate(binary_to_list(X), integer);
validate(X, string) when is_binary(X)   -> to_binary(X);
validate(X, string) when is_list(X)     -> to_binary(X);
validate(timestamp, date)               -> erlang:date();
validate(timestamp, time)               -> erlang:time();
validate(timestamp, datetime)           ->
  calendar:now_to_datetime(erlang:now());
validate(X, F) when is_function(F, 1)   -> F(X);
validate(X, RE) when is_binary(RE)      ->
  case re:run(X, RE, [{capture, none}]) of
    match   -> to_binary(X);
    nomatch -> erlang:error(no_match)
  end;
validate(X, date)                       -> str_to_date(to_list(X));
validate(X, time)                       -> str_to_time(to_list(X));
validate(X, datetime)                   -> str_to_datetime(to_list(X)).

types() -> [float, integer, string, date, time, datetime].

%% Outgoing requests
encode(Val, _) when is_boolean(Val)                -> to_binary(Val);
encode(undefined, _)                               -> <<"null">>;
encode(null, _)                                    -> <<"null">>;
encode(Val, Encoding) when is_list(Val)            ->
  unicode:characters_to_binary(Val, Encoding, utf8);
encode(Val, Encoding) when is_binary(Val)          ->
  unicode:characters_to_binary(Val, Encoding, utf8);
encode(Val, _) when is_integer(Val)                -> Val;
encode(Val, _) when is_float(Val)                  -> Val;
encode({datetime, {{Y, M, D}, {_H, _Min, _S}}}, _) ->
  list_to_binary(
    lists:flatten(io_lib:format("~4..0w-~2..0w-~2..0w", [Y, M, D])));
encode({date, {Y, M, D}}, _)                       ->
  list_to_binary(
    lists:flatten(io_lib:format("~4..0w-~2..0w-~2..0w", [Y, M, D])));
encode({time, {H, Min, S}}, _)                     ->
  list_to_binary(
    lists:flatten(io_lib:format("~2..0w:~2..0w:~2..0w", [H, Min, S])));
encode(Val, _)                                     ->
  {error, {unrecognized_value, Val}}.

%%%_* Time conversion --------------------------------------------------
%% "YYYY-MM-DD"
str_to_date(Date) ->
  {ok, [Year, Month, Day], []} = io_lib:fread("~4d-~2d-~2d", Date),
  {Year, Month, Day}.

%% "HH:MM:SS"
%% H,M,D have no size limit so either one or two chars will work
%% (if the num is lower than 10).
str_to_time(Time) ->
  {ok, [Hours, Minutes, Seconds], []} = io_lib:fread("~d:~d:~d", Time),
  {Hours, Minutes, Seconds}.

%% "YYYY-MM-DD HH:MM:SS"
%% H,M,D have no size limit so either one or two chars will work
%% (if the num is lower than 10).
str_to_datetime(Datetime) ->
  {ok, [Year, Month, Day, Hours, Minutes, Seconds], []} =
    io_lib:fread("~4d-~2d-~2d ~d:~d:~d", Datetime),
  {{Year, Month, Day}, {Hours, Minutes, Seconds}}.

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
